﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Data;
using System.Windows.Input;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ScheduleView;

namespace ScheduleView_WPF_ContexMenu
{
    public class ViewModel : ViewModelBase
    {
        private Action refreshAction;
        private ObservableCollection<Appointment> appointments;      
        private DelegateCommand setFreeCommand;
        private DelegateCommand setBusyCommand;
        private DelegateCommand setOutOfOfficeCommand;
        private DelegateCommand setTentativeCommand;     
        private bool isContextMenuOpen;

        public ViewModel(Action refreshAction)
        {
            this.refreshAction = refreshAction;
            this.Appointments = AddAppointments();

            this.SetFreeCommand = new DelegateCommand(this.SetFreeCommandExecuted, CanExecuteSetTimeMarker);
            this.SetBusyCommand = new DelegateCommand(this.SetBusyCommandExecuted, CanExecuteSetTimeMarker);
            this.SetOutOfOfficeCommand = new DelegateCommand(this.SetOutOfOfficeCommandExecuted, CanExecuteSetTimeMarker);
            this.SetTentativeCommand = new DelegateCommand(this.SetTentativeCommandExecuted, CanExecuteSetTimeMarker);      
        }

        public ObservableCollection<Appointment> Appointments
        {
            get
            {
                return this.appointments;
            }
            set
            {
                this.appointments = value;
            }
        }

        private ObservableCollection<Appointment> AddAppointments()
        {
            ObservableCollection<Appointment> appointments = new ObservableCollection<Appointment>();
            var app = new Appointment()
            {
                Start = DateTime.Now.AddHours(1),
                End = DateTime.Now.AddHours(2),
                Subject = "Test"
            };

            appointments.Add(app);
            return appointments;
        }    

        public DelegateCommand SetFreeCommand
        {
            get
            {
                return this.setFreeCommand;
            }
            set
            {
                this.setFreeCommand = value;
            }
        }

        public DelegateCommand SetBusyCommand
        {
            get
            {
                return this.setBusyCommand;
            }
            set
            {
                this.setBusyCommand = value;
            }
        }

        public DelegateCommand SetOutOfOfficeCommand
        {
            get
            {
                return this.setOutOfOfficeCommand;
            }
            set
            {
                this.setOutOfOfficeCommand = value;
            }
        }

        public DelegateCommand SetTentativeCommand
        {
            get
            {
                return this.setTentativeCommand;
            }
            set
            {
                this.setTentativeCommand = value;
            }
        }

        private static bool CanExecuteSetTimeMarker(object parameter)
        {
            IEnumerable appointments = parameter as IEnumerable;
            return appointments != null && appointments.OfType<Appointment>().Any();
        }

        private void CommandsInvalidateCanExecute()
        {
            this.SetFreeCommand.InvalidateCanExecute();
            this.SetBusyCommand.InvalidateCanExecute();
            this.SetOutOfOfficeCommand.InvalidateCanExecute();
            this.SetTentativeCommand.InvalidateCanExecute();
        
        }

        public bool IsContextMenuOpen
        {
            get
            {
                return this.isContextMenuOpen;
            }
            set
            {
                this.isContextMenuOpen = value;
                this.CommandsInvalidateCanExecute();
            }
        }

        private void SetFreeCommandExecuted(object parameter)
        {
            this.SetTimeMarker(parameter, TimeMarker.Free);
        }

        private void SetBusyCommandExecuted(object parameter)
        {
            this.SetTimeMarker(parameter, TimeMarker.Busy);
        }

        private void SetOutOfOfficeCommandExecuted(object parameter)
        {
            this.SetTimeMarker(parameter, TimeMarker.OutOfOffice);
        }

        private void SetTentativeCommandExecuted(object parameter)
        {
            this.SetTimeMarker(parameter, TimeMarker.Tentative);
        }   

        private void SetTimeMarker(object parameter, TimeMarker timeMarker)
        {
            IEnumerable appointments = parameter as IEnumerable;
            if (appointments != null)
            {
                foreach (Appointment appointment in appointments.OfType<Appointment>())
                {                           
                    appointment.TimeMarker = timeMarker;              
                }

                this.refreshAction();
            }
        }
    }
}

